#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/9/30 13:58
Desc: 期权配置文件
"""

import datetime
import json
import os
import re

# 中国金融期货交易所

CFFEX_OPTION_URL_300 = "http://www.cffex.com.cn/quote_IO.txt"

# 深圳证券交易所

SZ_OPTION_URL_300 = "http://www.szse.cn/api/report/ShowReport?SHOWTYPE=xlsx&CATALOGID=ysplbrb&TABKEY=tab1&random=0.10432465776720479"

# 上海证券交易所

SH_OPTION_URL_50 = "http://yunhq.sse.com.cn:32041/v1/sh1/list/self/510050"
SH_OPTION_URL_KING_50 = "http://yunhq.sse.com.cn:32041/v1/sho/list/tstyle/510050_{}"

SH_OPTION_URL_300 = "http://yunhq.sse.com.cn:32041/v1/sh1/list/self/510300"
SH_OPTION_URL_KING_300 = "http://yunhq.sse.com.cn:32041/v1/sho/list/tstyle/510300_{}"

SH_OPTION_URL_500 = "http://yunhq.sse.com.cn:32041/v1/sh1/list/self/510500"
SH_OPTION_URL_KING_500 = "http://yunhq.sse.com.cn:32041/v1/sho/list/tstyle/510500_{}"

SH_OPTION_URL_KC_50 = "http://yunhq.sse.com.cn:32041/v1/sh1/list/self/588000"
SH_OPTION_URL_KC_KING_50 = "http://yunhq.sse.com.cn:32041/v1/sho/list/tstyle/588000_{}"

SH_OPTION_URL_KC_50_YFD = "http://yunhq.sse.com.cn:32041/v1/sh1/list/self/588080"
SH_OPTION_URL_KING_50_YFD = "http://yunhq.sse.com.cn:32041/v1/sho/list/tstyle/588080_{}"

SH_OPTION_PAYLOAD = {
    "select": "select: code,name,last,change,chg_rate,amp_rate,volume,amount,prev_close"
}

SH_OPTION_PAYLOAD_OTHER = {"select": "contractid,last,chg_rate,presetpx,exepx"}


# 大连商品交易所
DCE_OPTION_URL = "http://portal.dce.com.cn/publicweb/quotesdata/dayQuotesCh.html"
DCE_DAILY_OPTION_URL = (
    "http://portal.dce.com.cn/publicweb/quotesdata/exportDayQuotesChData.html"
)

# 上海期货交易所
SHFE_OPTION_URL = "https://tsite.shfe.com.cn/data/dailydata/option/kx/kx{}.dat"

# 郑州商品交易所
CZCE_DAILY_OPTION_URL_3 = (
    "http://www.czce.com.cn/cn/DFSStaticFiles/Option/{}/{}/OptionDataDaily.txt"
)

# PAYLOAD
SHFE_HEADERS = {"User-Agent": "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)"}

DATE_PATTERN = re.compile(r"^([0-9]{4})[-/]?([0-9]{2})[-/]?([0-9]{2})")


def convert_date(date):
    """
    transform a date string to datetime.date object
    :param date, string, e.g. 2016-01-01, 20160101 or 2016/01/01
    :return: object of datetime.date(such as 2016-01-01) or None
    """
    if isinstance(date, datetime.date):
        return date
    elif isinstance(date, str):
        match = DATE_PATTERN.match(date)
        if match:
            groups = match.groups()
            if len(groups) == 3:
                return datetime.date(
                    year=int(groups[0]), month=int(groups[1]), day=int(groups[2])
                )
    return None


def get_json_path(name, module_file):
    """
    获取 JSON 配置文件的路径(从模块所在目录查找)
    :param name: 文件名
    :param module_file: filename
    :return: str json_file_path
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "file_fold", name)
    return module_json_path


def get_calendar():
    """
    获取交易日历至 2019 年结束, 这里的交易日历需要按年更新
    :return: json
    """
    setting_file_name = "calendar.json"
    setting_file_path = get_json_path(setting_file_name, __file__)
    return json.load(open(setting_file_path, "r"))


def last_trading_day(day):
    """
    获取前一个交易日
    :param day: "%Y%m%d" or  datetime.date()
    :return last_day: "%Y%m%d" or  datetime.date()
    """
    calendar = get_calendar()

    if isinstance(day, str):
        if day not in calendar:
            print("Today is not trading day：" + day)
            return False
        pos = calendar.index(day)
        last_day = calendar[pos - 1]
        return last_day

    elif isinstance(day, datetime.date):
        d_str = day.strftime("%Y%m%d")
        if d_str not in calendar:
            print("Today is not working day：" + d_str)
            return False
        pos = calendar.index(d_str)
        last_day = calendar[pos - 1]
        last_day = datetime.datetime.strptime(last_day, "%Y%m%d").date()
        return last_day


def get_latest_data_date(day):
    """
    获取最新的有数据的交易日
    :param day: datetime.datetime
    :return string YYYYMMDD
    """
    calendar = get_calendar()
    if day.strftime("%Y%m%d") in calendar:
        if day.time() > datetime.time(17, 0, 0):
            return day.strftime("%Y%m%d")
        else:
            return last_trading_day(day.strftime("%Y%m%d"))
    else:
        while day.strftime("%Y%m%d") not in calendar:
            day = day - datetime.timedelta(days=1)
        return day.strftime("%Y%m%d")


if __name__ == "__main__":
    d = datetime.datetime(2018, 10, 5, 17, 1, 0)
    print(get_latest_data_date(d))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/21 20:00
Desc: 同花顺-数据中心-宏观数据-股票筹资
https://data.10jqka.com.cn/macro/finance/
"""

from io import StringIO

import pandas as pd
import requests


def macro_stock_finance() -> pd.DataFrame:
    """
    同花顺-数据中心-宏观数据-股票筹资
    https://data.10jqka.com.cn/macro/finance/
    :return: 股票筹资
    :rtype: pandas.DataFrame
    """
    url = "https://data.10jqka.com.cn/macro/finance/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
    }
    r = requests.get(url, headers=headers)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.rename(
        columns={
            "月份": "月份",
            "募集资金(亿元)": "募集资金",
            "首发募集资金(亿元)": "首发募集资金",
            "增发募集资金(亿元)": "增发募集资金",
            "配股募集资金(亿元)": "配股募集资金",
        },
        inplace=True,
    )
    temp_df = temp_df[
        ["月份", "募集资金", "首发募集资金", "增发募集资金", "配股募集资金"]
    ]
    temp_df["募集资金"] = pd.to_numeric(temp_df["募集资金"], errors="coerce")
    temp_df["首发募集资金"] = pd.to_numeric(temp_df["首发募集资金"], errors="coerce")
    temp_df["增发募集资金"] = pd.to_numeric(temp_df["增发募集资金"], errors="coerce")
    temp_df["配股募集资金"] = pd.to_numeric(temp_df["配股募集资金"], errors="coerce")
    temp_df.sort_values(by=["月份"], inplace=True, ignore_index=True)
    return temp_df


def macro_rmb_loan() -> pd.DataFrame:
    """
    同花顺-数据中心-宏观数据-新增人民币贷款
    https://data.10jqka.com.cn/macro/loan/
    :return: 新增人民币贷款
    :rtype: pandas.DataFrame
    """
    url = "https://data.10jqka.com.cn/macro/loan/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
    }
    r = requests.get(url, headers=headers)
    temp_df = pd.read_html(StringIO(r.text), skiprows=0)[0]
    temp_df.columns = [
        "月份",
        "新增人民币贷款-总额",
        "新增人民币贷款-同比",
        "新增人民币贷款-环比",
        "累计人民币贷款-总额",
        "累计人民币贷款-同比",
    ]
    temp_df["新增人民币贷款-总额"] = pd.to_numeric(
        temp_df["新增人民币贷款-总额"], errors="coerce"
    )
    temp_df["累计人民币贷款-总额"] = pd.to_numeric(
        temp_df["累计人民币贷款-总额"], errors="coerce"
    )
    temp_df.sort_values(by=["月份"], inplace=True, ignore_index=True)
    return temp_df


def macro_rmb_deposit() -> pd.DataFrame:
    """
    同花顺-数据中心-宏观数据-人民币存款余额
    https://data.10jqka.com.cn/macro/rmb/
    :return: 人民币存款余额
    :rtype: pandas.DataFrame
    """
    url = "https://data.10jqka.com.cn/macro/rmb/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36"
    }
    r = requests.get(url, headers=headers)
    temp_df = pd.read_html(StringIO(r.text), skiprows=0)[0]
    temp_df.columns = [
        "月份",
        "新增存款-数量",
        "新增存款-同比",
        "新增存款-环比",
        "新增企业存款-数量",
        "新增企业存款-同比",
        "新增企业存款-环比",
        "新增储蓄存款-数量",
        "新增储蓄存款-同比",
        "新增储蓄存款-环比",
        "新增其他存款-数量",
        "新增其他存款-同比",
        "新增其他存款-环比",
    ]
    temp_df["新增存款-数量"] = pd.to_numeric(temp_df["新增存款-数量"], errors="coerce")
    temp_df["新增企业存款-数量"] = pd.to_numeric(
        temp_df["新增企业存款-数量"], errors="coerce"
    )
    temp_df["新增企业存款-数量"] = pd.to_numeric(
        temp_df["新增企业存款-数量"], errors="coerce"
    )
    temp_df["新增储蓄存款-数量"] = pd.to_numeric(
        temp_df["新增储蓄存款-数量"], errors="coerce"
    )
    temp_df["新增其他存款-数量"] = pd.to_numeric(
        temp_df["新增其他存款-数量"], errors="coerce"
    )
    temp_df.sort_values(by=["月份"], inplace=True, ignore_index=True)
    return temp_df


if __name__ == "__main__":
    macro_stock_finance_df = macro_stock_finance()
    print(macro_stock_finance_df)

    macro_rmb_loan_df = macro_rmb_loan()
    print(macro_rmb_loan_df)

    macro_rmb_deposit_df = macro_rmb_deposit()
    print(macro_rmb_deposit_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/2/22 15:45
Desc: 金十数据-期货手续费
https://www.jin10.com/
"""
import json

import pandas as pd
import requests


def futures_comm_js(date: str = "20260213") -> pd.DataFrame:
    """
    金十财经-期货手续费
    https://www.jin10.com/
    :param date: 日期; 格式为 YYYYMMDD，例如 "20250213"
    :type date: str
    :return: 期货手续费数据
    :rtype: pandas.DataFrame
    """
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "fiXF2nOnDycGutVA",
        "x-version": "1.0",
        "referer": "https://www.jin10.com/",
        "origin": "https://www.jin10.com",
    }
    url = "https://mp-api.jin10.com/api/dynamic-data/child"
    formatted_date = "-".join([date[:4], date[4:6], date[6:]])
    params = {
        "tb_name": "_vir_26",
        "search": json.dumps({"range,date": f"{formatted_date},{formatted_date}", "status": 1}),
        "order": "date,desc",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df = temp_df.rename(
        columns={
            "date": "日期",
            "heyue_name": "合约品种",
            "heyue_code": "合约代码",
            "pub_date_commission": "手续费公布时间",
            "pub_date_price": "价格公布时间",
            "heyue_price": "现价",
            "up_limit_num": "涨停板",
            "down_limit_num": "跌停板",
            "buy_ratio": "保证金/买开",
            "sell_ratio": "保证金/卖开",
            "per_lot_price": "保证金/每手",
            "per_ratio": "每手跳数",
            "buy_commission": "开仓",
            "sell_yesterday_commission": "平昨",
            "sell_cur_commission": "平今",
            "per_commission_price": "每跳毛利",
            "per_net_profit": "每跳净利",
            "jys": "交易所",
        }
    )
    temp_df = temp_df.drop(
        columns=["id", "status", "created_at", "updated_at", "_date", "_pub_date_commission", "_pub_date_price"],
        errors="ignore")
    temp_df['日期'] = pd.to_datetime(temp_df['日期'], errors="coerce").dt.date
    temp_df['现价'] = pd.to_numeric(temp_df['现价'], errors="coerce")
    temp_df['涨停板'] = pd.to_numeric(temp_df['涨停板'], errors="coerce")
    temp_df['跌停板'] = pd.to_numeric(temp_df['跌停板'], errors="coerce")
    temp_df['每手跳数'] = pd.to_numeric(temp_df['每手跳数'], errors="coerce")
    temp_df['每跳毛利'] = pd.to_numeric(temp_df['每跳毛利'], errors="coerce")
    temp_df['每跳净利'] = pd.to_numeric(temp_df['每跳净利'], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_comm_js_df = futures_comm_js(date="20260213")
    print(futures_comm_js_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/12/26 18:30
Desc: 商品现货价格指数
https://finance.sina.com.cn/futuremarket/spotprice.shtml#titlePos_0
"""

import pandas as pd
import requests


def spot_goods(symbol: str = "波罗的海干散货指数") -> pd.DataFrame:
    """
    新浪财经-商品现货价格指数
    https://finance.sina.com.cn/futuremarket/spotprice.shtml#titlePos_0
    :param symbol: choice of {"波罗的海干散货指数", "钢坯价格指数", "澳大利亚粉矿价格"}
    :type symbol: str
    :return: 商品现货价格指数
    :rtype: pandas.DataFrame
    """
    url = "https://stock.finance.sina.com.cn/futures/api/openapi.php/GoodsIndexService.get_goods_index"
    symbol_url_dict = {
        "波罗的海干散货指数": "BDI",
        "钢坯价格指数": "GP",
        "澳大利亚粉矿价格": "PB",
    }
    params = {"symbol": symbol_url_dict[symbol], "table": "0"}
    r = requests.get(url, params=params)
    r.encoding = "gbk"
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"]["data"])
    temp_df = temp_df[["opendate", "price", "zde", "zdf"]]
    temp_df.columns = ["日期", "指数", "涨跌额", "涨跌幅"]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["指数"] = pd.to_numeric(temp_df["指数"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df.dropna(inplace=True, ignore_index=True)
    return temp_df


if __name__ == "__main__":
    spot_goods_df = spot_goods(symbol="波罗的海干散货指数")
    print(spot_goods_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/2/20 10:00
Desc: 期货结算信息
期货交易所结算参数数据
- 中金所: 结算参数(保证金、手续费等) - 已实现
- 郑商所: 结算参数 - 已实现
- 上期所: 结算参数 - 已实现
- 广期所: 结算参数 - 已实现
- 上能中心: 结算参数 - 已实现
- 大商所: 待解决(网站反爬虫保护，所有接口返回412错误)
"""

import datetime

import pandas as pd
import requests
from io import StringIO

from akshare.futures import cons
from akshare.utils.cons import headers

gfex_headers = {
    "Accept": "application/json, text/javascript, */*; q=0.01",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
    "Origin": "http://www.gfex.com.cn",
    "Referer": "http://www.gfex.com.cn/gfex/rjycs/ywcs.shtml",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/144.0.0.0 Safari/537.36",
    "X-Requested-With": "XMLHttpRequest",
}

# 统一的结算参数字段
SETTLE_OUTPUT_COLUMNS = [
    "date",
    "symbol",
    "variety",
    "settle_price",
    "long_margin_ratio",
    "short_margin_ratio",
    "spec_long_margin_ratio",
    "spec_short_margin_ratio",
    "hedge_long_margin_ratio",
    "hedge_short_margin_ratio",
    "trade_fee_ratio",
    "close_today_fee_ratio",
    "delivery_fee_ratio",
    "is_single_market",
    "single_market_days",
    "limit_ratio",
    "position_limit",
    "trade_limit",
    "rise_limit_rate",
    "fall_limit_rate",
]


def _normalize_settle_columns(df: pd.DataFrame) -> pd.DataFrame:
    """
    统一结算参数字段，将各交易所的字段映射到统一字段
    :param df: 原始DataFrame
    :type df: pandas.DataFrame
    :return: 统一格式的DataFrame
    :rtype: pandas.DataFrame
    """
    if df.empty:
        return pd.DataFrame(columns=SETTLE_OUTPUT_COLUMNS)

    # 字段映射关系
    field_mapping = {
        # 统一字段 -> 可能的来源字段
        "settle_price": ["settle_price", "SETTLEMENTPRICE", "当日结算价"],
        "long_margin_ratio": ["long_margin_ratio", "margin_ratio", "SPECLONGMARGINRATIO", "specBuyRate", "spec_buy_rate"],
        "short_margin_ratio": ["short_margin_ratio", "SPECSHORTMARGINRATIO", "hedgeBuyRate", "hedge_buy_rate"],
        "spec_long_margin_ratio": ["spec_long_margin_ratio", "SPECLONGMARGINRATIO", "spec_buy_rate"],
        "spec_short_margin_ratio": ["spec_short_margin_ratio", "SPECSHORTMARGINRATIO", "hedge_buy_rate"],
        "hedge_long_margin_ratio": ["hedge_long_margin_ratio", "HEDGLONGMARGINRATIO", "hedge_buy_rate"],
        "hedge_short_margin_ratio": ["hedge_short_margin_ratio", "HEDGSHORTMARGINRATIO", "spec_buy_rate"],
        "trade_fee_ratio": ["trade_fee_ratio", "TRADEFEERATIO", "交易手续费"],
        "close_today_fee_ratio": ["close_today_fee_ratio", "TTRADEFEERATIO", "日内平今仓交易手续费"],
        "delivery_fee_ratio": ["delivery_fee_ratio", "COMMODITYDELIVFEERATIO", "交割手续费"],
        "is_single_market": ["is_single_market", "是否单边市"],
        "single_market_days": ["single_market_days", "连续单边市天数"],
        "limit_ratio": ["limit_ratio", "涨跌停板(%)"],
        "position_limit": ["position_limit", "日持仓限额", "clientBuyPosiQuota", "client_buy_posi_quota"],
        "trade_limit": ["trade_limit", "交易限额"],
        "rise_limit_rate": ["rise_limit_rate", "riseLimitRate", "rise_limit_rate"],
        "fall_limit_rate": ["fall_limit_rate", "fallLimit", "fall_limit"],
    }

    # 确保必要的字段存在
    for col in ["date", "symbol", "variety"]:
        if col not in df.columns:
            if col == "variety" and "symbol" in df.columns:
                df["variety"] = df["symbol"].str.extract(r"([A-Za-z]+)", expand=False)
            else:
                df[col] = None

    # 映射字段
    for target_field, possible_sources in field_mapping.items():
        if target_field not in df.columns:
            for source in possible_sources:
                if source in df.columns:
                    df[target_field] = df[source]
                    break
            else:
                df[target_field] = None

    # 返回统一格式
    return df[SETTLE_OUTPUT_COLUMNS]


def _parse_pipe_data(text: str) -> pd.DataFrame:
    """
    解析管道符分隔的数据
    :param text: 原始文本数据
    :type text: str
    :return: 解析后的DataFrame
    :rtype: pandas.DataFrame
    """
    lines = text.strip().split("\n")
    if len(lines) < 2:
        return pd.DataFrame()
    columns = [col.strip() for col in lines[1].split("|")]
    data_lines = [line for line in lines[2:] if line.strip()]
    data_list = []
    for line in data_lines:
        row = [col.strip() for col in line.split("|")]
        if len(row) >= len(columns):
            data_list.append(row[:len(columns)])
    return pd.DataFrame(data_list, columns=columns)


def futures_settle_cffex(date: str = "20260119") -> pd.DataFrame:
    """
    中国金融期货交易所-结算参数
    http://www.cffex.com.cn/jscs/
    :param date: 结算参数日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 结算参数数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    date = day.strftime("%Y%m%d")
    url = f"http://www.cffex.com.cn/sj/jscs/{date[:4]}{date[4:6]}/{date[6:8]}/{date}_1.csv"
    r = requests.get(url, headers=headers)
    r.encoding = "gbk"
    if r.status_code != 200:
        return pd.DataFrame()
    # 检查是否返回的是 HTML 页面（页面不存在或数据未发布）
    if r.text.strip().startswith("<") or "要查看的页面不存在" in r.text:
        return pd.DataFrame()
    try:
        data_df = pd.read_csv(StringIO(r.text), skiprows=1)
    except:  # noqa: E722
        return pd.DataFrame()
    if data_df.shape[0] < 5:
        return pd.DataFrame()
    data_df.columns = [
        "symbol",
        "long_margin_ratio",
        "short_margin_ratio",
        "trade_fee_ratio",
        "delivery_fee_ratio",
        "close_today_fee_ratio",
    ]
    data_df = data_df[data_df["symbol"].notna()]
    data_df = data_df[data_df["symbol"].str.contains(
        r"^[A-Z]+", na=False, regex=True)]
    data_df["variety"] = data_df["symbol"].str.extract(r"([A-Z]+)")
    data_df["date"] = date
    data_df = data_df[
        ["date", "symbol", "variety", "long_margin_ratio", "short_margin_ratio",
         "trade_fee_ratio", "delivery_fee_ratio", "close_today_fee_ratio"]
    ]
    return data_df


def futures_settle_czce(date: str = "20260119") -> pd.DataFrame:
    """
    郑州商品交易所-结算参数
    http://www.czce.com.cn/cn/jysj/jscs/H077003003index_1.htm
    :param date: 结算参数日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 结算参数数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    date = day.strftime("%Y%m%d")
    url = f"http://www.czce.com.cn/cn/DFSStaticFiles/Future/{date[:4]}/{date}/FutureDataClearParams.txt"
    r = requests.get(url, headers=headers)
    r.encoding = "utf-8"
    if r.status_code != 200:
        return pd.DataFrame()
    try:
        data_df = _parse_pipe_data(r.text)
    except:  # noqa: E722
        return pd.DataFrame()
    if data_df.shape[0] < 5:
        return pd.DataFrame()
    data_df.columns = [
        "symbol", "settle_price", "is_single_market", "single_market_days",
        "margin_ratio", "limit_ratio", "trade_fee", "fee_type",
        "delivery_fee", "close_today_fee", "position_limit", "trade_limit"
    ]
    data_df = data_df[data_df["symbol"].notna()]
    data_df = data_df[~data_df["symbol"].str.contains("小计|合计|总计", na=False)]
    data_df["variety"] = data_df["symbol"].str.extract(r"([A-Za-z]+)")
    data_df["date"] = date
    data_df = data_df[
        ["date", "symbol", "variety", "settle_price", "is_single_market", "single_market_days",
         "margin_ratio", "limit_ratio", "trade_fee", "fee_type", "delivery_fee",
         "close_today_fee", "position_limit", "trade_limit"]
    ]
    return data_df


def futures_settle_gfex(date: str = "20260119") -> pd.DataFrame:
    """
    广州期货交易所-结算参数
    http://www.gfex.com.cn/gfex/rjycs/ywcs.shtml
    :param date: 结算参数日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 结算参数数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    date = day.strftime("%Y%m%d")
    url = "http://www.gfex.com.cn/u/interfacesWebTtQueryTradPara/loadDayList"
    payload = {"trade_type": "0"}
    r = requests.post(url, data=payload, headers=gfex_headers)
    if r.status_code != 200:
        return pd.DataFrame()
    # 检查是否返回了反爬虫的 JavaScript 代码
    if r.text.strip().startswith("<script") or "function" in r.text[:100]:
        return pd.DataFrame()
    try:
        json_data = r.json()
        if json_data.get("code") != "0":
            return pd.DataFrame()
        data_list = json_data.get("data", [])
    except:  # noqa: E722
        return pd.DataFrame()
    if not data_list:
        return pd.DataFrame()
    # 过滤掉期权合约，只保留期货合约
    data_list = [
        item for item in data_list if "-" not in item.get("contractId", "")]
    if not data_list:
        return pd.DataFrame()
    data_df = pd.DataFrame(data_list)
    data_df.columns = [
        "symbol", "spec_buy_rate", "spec_buy", "hedge_buy_rate", "hedge_buy",
        "rise_limit_rate", "rise_limit", "fall_limit", "agent_tot_buy_posi_quota",
        "self_tot_buy_posi_quota", "client_buy_posi_quota", "self_tot_buy_ser_limit",
        "client_buy_ser_limit", "trade_type"
    ]
    data_df["variety"] = data_df["symbol"].str.extract(r"([A-Za-z]+)")
    data_df["date"] = date
    data_df = data_df[
        ["date", "symbol", "variety", "spec_buy_rate", "spec_buy", "hedge_buy_rate",
         "hedge_buy", "rise_limit_rate", "rise_limit", "fall_limit",
         "agent_tot_buy_posi_quota", "self_tot_buy_posi_quota", "client_buy_posi_quota"]
    ]
    return data_df


def futures_settle_shfe(date: str = "20260119") -> pd.DataFrame:
    """
    上海期货交易所-结算参数
    https://www.shfe.com.cn/reports/tradedata/dailyandweeklydata/
    :param date: 结算参数日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 结算参数数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    date = day.strftime("%Y%m%d")
    url = f"https://www.shfe.com.cn/data/tradedata/future/dailydata/js{date}.dat"
    r = requests.get(url, headers=cons.shfe_headers)
    if r.status_code != 200:
        return pd.DataFrame()
    try:
        data_json = r.json()
        data_list = data_json.get("o_cursor", [])
    except:  # noqa: E722
        return pd.DataFrame()
    if not data_list:
        return pd.DataFrame()
    data_df = pd.DataFrame(data_list)
    data_df.columns = [
        "symbol", "trade_fee_ratio", "close_today_fee_ratio", "delivery_fee_unit",
        "spec_long_margin_ratio", "hedge_long_margin_ratio", "delivery_fee_ratio",
        "product_id", "product_name", "close_today_fee_unit", "trade_fee_unit",
        "hedge_short_margin_ratio", "settle_price", "uni_direction",
        "spec_short_margin_ratio", "is_close_today"
    ]
    data_df["variety"] = data_df["symbol"].str.extract(r"([A-Za-z]+)")
    data_df["date"] = date
    data_df = data_df[
        ["date", "symbol", "variety", "settle_price", "spec_long_margin_ratio",
         "hedge_long_margin_ratio", "spec_short_margin_ratio", "hedge_short_margin_ratio",
         "trade_fee_ratio", "close_today_fee_ratio", "is_close_today"]
    ]
    return data_df


def futures_settle_ine(date: str = "20260119") -> pd.DataFrame:
    """
    上海国际能源交易中心-结算参数
    https://www.ine.cn/reports/businessdata/prmsummary/
    :param date: 结算参数日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :return: 结算参数数据
    :rtype: pandas.DataFrame
    """
    day = cons.convert_date(date) if date is not None else datetime.date.today()
    date = day.strftime("%Y%m%d")
    url = f"https://www.ine.cn/data/tradedata/future/dailydata/js{date}.dat"
    r = requests.get(url, headers=cons.shfe_headers)
    if r.status_code != 200:
        return pd.DataFrame()
    try:
        data_json = r.json()
        data_list = data_json.get("o_cursor", [])
    except:  # noqa: E722
        return pd.DataFrame()
    if not data_list:
        return pd.DataFrame()
    data_df = pd.DataFrame(data_list)
    data_df.columns = [
        "symbol", "trade_fee_ratio", "close_today_fee_ratio", "delivery_fee_unit",
        "spec_long_margin_ratio", "hedge_long_margin_ratio", "delivery_fee_ratio",
        "product_id", "product_name", "close_today_fee_unit", "trade_fee_unit",
        "hedge_short_margin_ratio", "settle_price", "uni_direction",
        "spec_short_margin_ratio", "is_close_today"
    ]
    data_df["variety"] = data_df["symbol"].str.extract(r"([A-Za-z]+)")
    data_df["date"] = date
    data_df = data_df[
        ["date", "symbol", "variety", "settle_price", "spec_long_margin_ratio",
         "hedge_long_margin_ratio", "spec_short_margin_ratio", "hedge_short_margin_ratio",
         "trade_fee_ratio", "close_today_fee_ratio", "is_close_today"]
    ]
    return data_df


def futures_settle(date: str = "20260119", market: str = "CFFEX") -> pd.DataFrame:
    """
    期货交易所结算参数
    :param date: 结算日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象，默认为当前交易日
    :type date: str or datetime.date
    :param market: 交易所代码: CFFEX-中金所, CZCE-郑商所, SHFE-上期所, DCE-大商所, INE-上能中心, GFEX-广期所
    :type market: str
    :return: 结算参数数据（统一格式）
    :rtype: pandas.DataFrame
    """
    if market.upper() == "CFFEX":
        df = futures_settle_cffex(date)
    elif market.upper() == "CZCE":
        df = futures_settle_czce(date)
    elif market.upper() == "SHFE":
        df = futures_settle_shfe(date)
    elif market.upper() == "GFEX":
        df = futures_settle_gfex(date)
    elif market.upper() == "INE":
        df = futures_settle_ine(date)
    else:
        print(f"Unsupported market: {market}")
        return pd.DataFrame(columns=SETTLE_OUTPUT_COLUMNS)
    temp_df = _normalize_settle_columns(df)
    return temp_df


if __name__ == "__main__":
    futures_settle_cffex_df = futures_settle_cffex(date="20260119")
    print("=== 中金所结算参数 ===")
    print(futures_settle_cffex_df)

    futures_settle_czce_df = futures_settle_czce(date="20260119")
    print("\n=== 郑商所结算参数 ===")
    print(futures_settle_czce_df)

    futures_settle_shfe_df = futures_settle_shfe(date="20260119")
    print("\n=== 上期所结算参数 ===")
    print(futures_settle_shfe_df)

    futures_settle_gfex_df = futures_settle_gfex(date="20260119")
    print("\n=== 广期所结算参数 ===")
    print(futures_settle_gfex_df)

    futures_settle_ine_df = futures_settle_ine(date="20250117")
    print("\n=== 上能中心结算参数 ===")
    print(futures_settle_ine_df)

    futures_settle_df = futures_settle(date="20260119", market="CFFEX")
    print("\n=== 通用接口-CFFEX ===")
    print(futures_settle_df)

    futures_settle_df = futures_settle(date="20260119", market="CZCE")
    print("\n=== 通用接口-CZCE ===")
    print(futures_settle_df)

    futures_settle_df = futures_settle(date="20260119", market="SHFE")
    print("\n=== 通用接口-SHFE ===")
    print(futures_settle_df)

    futures_settle_df = futures_settle(date="20260119", market="GFEX")
    print("\n=== 通用接口-GFEX ===")
    print(futures_settle_df)

    futures_settle_df = futures_settle(date="20260119", market="INE")
    print("\n=== 通用接口-INE ===")
    print(futures_settle_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/2/4 17:00
Desc: 新浪财经-债券-中国/美国国债收益率
https://vip.stock.finance.sina.com.cn/mkt/#hs_z
"""

import pandas as pd
import requests


def bond_gb_zh_sina(symbol: str = "中国10年期国债") -> pd.DataFrame:
    """
    新浪财经-债券-中国国债收益率行情数据
    https://stock.finance.sina.com.cn/forex/globalbd/cn10yt.html
    :param symbol: choice of {"中国1年期国债", "中国2年期国债", "中国3年期国债", "中国5年期国债", "中国7年期国债", "中国10年期国债", "中国15年期国债", "中国20年期国债", "中国30年期国债"}
    :type symbol: str
    :return: 中国国债收益率行情数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "中国1年期国债": "CN1YT",
        "中国2年期国债": "CN2YT",
        "中国3年期国债": "CN3YT",
        "中国5年期国债": "CN5YT",
        "中国7年期国债": "CN7YT",
        "中国10年期国债": "CN10YT",
        "中国15年期国债": "CN15YT",
        "中国20年期国债": "CN20YT",
        "中国30年期国债": "CN30YT",
    }
    url = f"https://bond.finance.sina.com.cn/hq/gb/daily?symbol={symbol_map[symbol]}"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
        "volume",
    ]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    return temp_df


def bond_gb_us_sina(symbol: str = "美国10年期国债") -> pd.DataFrame:
    """
    新浪财经-债券-美国国债收益率行情数据
    https://stock.finance.sina.com.cn/forex/globalbd/cn10yt.html
    :param symbol: choice of {"美国1月期国债", "美国2月期国债", "美国3月期国债", "美国4月期国债", "美国6月期国债", "美国1年期国债", "美国2年期国债", "美国3年期国债", "美国5年期国债", "美国7年期国债", "美国10年期国债", "美国20年期国债", "美国30年期国债"}
    :type symbol: str
    :return: 美国国债收益率行情数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "美国1月期国债": "US1MT",
        "美国2月期国债": "US2MT",
        "美国3月期国债": "US3MT",
        "美国4月期国债": "US4MT",
        "美国6月期国债": "US6MT",
        "美国1年期国债": "US1YT",
        "美国2年期国债": "US2YT",
        "美国3年期国债": "US3YT",
        "美国5年期国债": "US5YT",
        "美国7年期国债": "US7YT",
        "美国10年期国债": "US10YT",
        "美国20年期国债": "US20YT",
        "美国30年期国债": "US30YT",
    }
    url = f"https://bond.finance.sina.com.cn/hq/gb/daily?symbol={symbol_map[symbol]}"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
        "volume",
    ]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    bond_gb_zh_sina_df = bond_gb_zh_sina(symbol="中国10年期国债")
    print(bond_gb_zh_sina_df)

    bond_gb_us_sina_df = bond_gb_us_sina(symbol="美国10年期国债")
    print(bond_gb_us_sina_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/12/26 18:00
Desc: 上海金属网-快讯
https://www.shmet.com/newsFlash/newsFlash.html?searchKeyword=
"""

import pandas as pd
import requests


def futures_news_shmet(symbol: str = "全部") -> pd.DataFrame:
    """
    上海金属网-快讯
    https://www.shmet.com/newsFlash/newsFlash.html?searchKeyword=
    :param symbol: choice of {"全部", "要闻", "VIP", "财经", "铜", "铝", "铅", "锌", "镍", "锡", "贵金属", "小金属"}
    :type symbol: str
    :return: 上海金属网-快讯
    :rtype: pandas.DataFrame
    """
    url = "https://www.shmet.com/api/rest/news/queryNewsflashList"
    if symbol == "全部":
        payload = {"currentPage": 1, "pageSize": 100}
    else:
        symbol_map = {
            "要闻": "0",
            "VIP": "100",
            "财经": "999",
            "铜": "1002",
            "铝": "1003",
            "铅": "1005",
            "锌": "1004",
            "镍": "1006",
            "锡": "1007",
            "贵金属": "1008",
            "小金属": "1009",
        }
        payload = {
            "currentPage": 1,
            "pageSize": 2000,
            "content": "",
            "flashTag": symbol_map[symbol],
        }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["dataList"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "发布时间",
        "-",
        "内容",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "发布时间",
            "内容",
        ]
    ]
    temp_df["发布时间"] = pd.to_datetime(
        temp_df["发布时间"], unit="ms", utc=True
    ).dt.tz_convert("Asia/Shanghai")
    temp_df.sort_values(["发布时间"], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


if __name__ == "__main__":
    futures_news_shmet_df = futures_news_shmet(symbol="铜")
    print(futures_news_shmet_df)

    for item in [
        "全部",
        "要闻",
        "VIP",
        "财经",
        "铜",
        "铝",
        "铅",
        "锌",
        "镍",
        "锡",
        "贵金属",
        "小金属",
    ]:
        futures_news_shmet_df = futures_news_shmet(symbol=item)
        print(futures_news_shmet_df)



"""
Yang-Zhang-s-Realized-Volatility-Automated-Estimation-in-Python
https://github.com/hugogobato/Yang-Zhang-s-Realized-Volatility-Automated-Estimation-in-Python
论文地址：https://www.jstor.org/stable/10.1086/209650
"""

import warnings

import numpy as np
import pandas as pd


def rv_from_stock_zh_a_hist_min_em(
    symbol="000001",
    start_date="2021-10-20 09:30:00",
    end_date="2024-11-01 15:00:00",
    period="1",
    adjust="hfq",
) -> pd.DataFrame:
    """
    从东方财富网获取股票的分钟级历史行情数据,并进行数据清洗和格式化为计算 yz 已实现波动率所需的数据格式
    https://quote.eastmoney.com/concept/sh603777.html?from=classic
    :param symbol: 股票代码,如"000001"
    :type symbol: str
    :param start_date: 开始日期时间,格式"YYYY-MM-DD HH:MM:SS"
    :type start_date: str
    :param end_date: 结束日期时间,格式"YYYY-MM-DD HH:MM:SS"
    :type end_date: str
    :param period: 时间周期,可选{'1','5','15','30','60'}分钟
    :type period: str
    :param adjust: 复权方式,可选{'','qfq'(前复权),'hfq'(后复权)}
    :type adjust: str
    :return: 整理后的分钟行情数据,包含Date(索引),Open,High,Low,Close列
    :rtype: pandas.DataFrame
    """
    from akshare.stock_feature.stock_hist_em import stock_zh_a_hist_min_em

    temp_df = stock_zh_a_hist_min_em(
        symbol=symbol,
        start_date=start_date,
        end_date=end_date,
        period=period,
        adjust=adjust,
    )
    temp_df.rename(
        columns={
            "时间": "Date",
            "开盘": "Open",
            "最高": "High",
            "最低": "Low",
            "收盘": "Close",
        },
        inplace=True,
    )
    temp_df = temp_df[temp_df["Open"] != 0]
    temp_df["Date"] = pd.to_datetime(temp_df["Date"])
    temp_df.set_index(keys="Date", inplace=True)
    return temp_df


def rv_from_futures_zh_minute_sina(
    symbol: str = "IF2008", period: str = "5"
) -> pd.DataFrame:
    """
    从新浪财经获取期货的分钟级历史行情数据,并进行数据清洗和格式化
    https://vip.stock.finance.sina.com.cn/quotes_service/view/qihuohangqing.html#titlePos_3
    :param symbol: 期货合约代码,如"IF2008"代表沪深300期货2020年8月合约
    :type symbol: str
    :param period: 时间周期,可选{'1','5','15','30','60'}分钟
    :type period: str
    :return: 整理后的分钟行情数据,包含Date(索引),Open,High,Low,Close列
    :rtype: pandas.DataFrame
    """
    from akshare.futures.futures_zh_sina import futures_zh_minute_sina

    temp_df = futures_zh_minute_sina(symbol=symbol, period=period)
    temp_df.rename(
        columns={
            "datetime": "Date",
            "open": "Open",
            "high": "High",
            "low": "Low",
            "close": "Close",
        },
        inplace=True,
    )
    temp_df["Date"] = pd.to_datetime(temp_df["Date"])
    temp_df.set_index(keys="Date", inplace=True)
    return temp_df


def volatility_yz_rv(data: pd.DataFrame) -> pd.DataFrame:
    (
        """
    波动率-已实现波动率-Yang-Zhang 已实现波动率(Yang-Zhang Realized Volatility)
    https://github.com/hugogobato/Yang-Zhang-s-Realized-Volatility-Automated-Estimation-in-Python
    论文地址：https://www.jstor.org/stable/10.1086/209650
    基于以下公式计算:
    RV^2 = Vo + k*Vc + (1-k)*Vrs
    其中:
    - Vo: 隔夜波动率, Vo = 1/(n-1)*sum(Oi-Obar)^2
        Oi为标准化开盘价, Obar为标准化开盘价均值
    - Vc: 收盘波动率, Vc = 1/(n-1)*sum(ci-Cbar)^2
        ci为标准化收盘价, Cbar为标准化收盘价均值
    - k: 权重系数, k = 0.34/(1.34+(n+1)/(n-1))
        n为样本数量
    - Vrs: Rogers-Satchell波动率代理, Vrs = ui(ui-ci)+di(di-ci)
        ui = ln(Hi/Oi), ci = ln(Ci/Oi), di = ln(Li/Oi), oi = ln(Oi/Ci-1)
        Hi/Li/Ci/Oi分别为最高价/最低价/收盘价/开盘价

    :param data: 包含 OHLC(开高低收) 价格的 pandas.DataFrame
    :type data: pandas.DataFrame
    :return: 包含 Yang-Zhang 实现波动率的 pandas.DataFrame
    :rtype: pandas.DataFrame

    要求输入数据包含以下列:
    - Open: 开盘价
    - High: 最高价
    - Low: 最低价
    - Close: 收盘价
    # yang_zhang_rv formula is give as:
    # RV^2 = Vo + k*Vc + (1-k)*Vrs
    # where Vo = 1/(n-1)*sum(Oi-Obar)^2
    # with oi = normalized opening price at time t and Obar = mean of normalized opening prices
    # Vc = = 1/(n-1)*sum(ci-Cbar)^2
    # with ci = normalized close price at time t and Cbar = mean of normalized close prices
    # k = 0.34/(1.34+(n+1)/(n-1))
    # with n = total number of days or time periods considered
    # Vrs (Rogers & Satchell RV proxy) = ui(ui-ci)+di(di-ci)
    # with ui = ln(Hi/Oi), ci = ln(Ci/Oi), di=(Li/Oi), oi = ln(Oi/Ci-1)
    # where Hi = high price at time t and Li = low price at time t
    """
        ""
    )
    warnings.filterwarnings("ignore")

    data["ui"] = np.log(np.divide(data["High"][1:], data["Open"][1:]))
    data["ci"] = np.log(np.divide(data["Close"][1:], data["Open"][1:]))
    data["di"] = np.log(np.divide(data["Low"][1:], data["Open"][1:]))
    data["oi"] = np.log(np.divide(data["Open"][1:], data["Close"][: len(data) - 1]))
    data = data[1:]
    data["RS"] = data["ui"] * (data["ui"] - data["ci"]) + data["di"] * (
        data["di"] - data["ci"]
    )
    rs_var = data["RS"].groupby(pd.Grouper(freq="1D")).mean().dropna()
    vc_and_vo = data[["oi", "ci"]].groupby(pd.Grouper(freq="1D")).var().dropna()
    n = int(len(data) / len(rs_var))
    k = 0.34 / (1.34 + (n + 1) / (n - 1))
    yang_zhang_rv = np.sqrt((1 - k) * rs_var + vc_and_vo["oi"] + vc_and_vo["ci"] * k)
    yang_zhang_rv_df = pd.DataFrame(yang_zhang_rv)
    yang_zhang_rv_df.rename(columns={0: "yz_rv"}, inplace=True)
    yang_zhang_rv_df.reset_index(inplace=True)
    yang_zhang_rv_df.columns = ["date", "rv"]
    yang_zhang_rv_df["date"] = pd.to_datetime(
        yang_zhang_rv_df["date"], errors="coerce"
    ).dt.date
    return yang_zhang_rv_df


if __name__ == "__main__":
    futures_df = rv_from_futures_zh_minute_sina(symbol="IF2008", period="1")
    volatility_yz_rv_df = volatility_yz_rv(data=futures_df)
    print(volatility_yz_rv_df)

    stock_df = rv_from_stock_zh_a_hist_min_em(
        symbol="000001",
        start_date="2021-10-20 09:30:00",
        end_date="2024-11-01 15:00:00",
        period="5",
        adjust="",
    )
    volatility_yz_rv_df = volatility_yz_rv(data=stock_df)
    print(volatility_yz_rv_df)
